@import url("resource:///org/gnome/builder/themes/shared.css");

idelayoutstackheader > button:last-child {
  margin-right: 3px;
}

entry.search-missing {
  background-color: #cc0000;
  color: white;
  text-shadow: none;
}

entry.search-missing > image {
  color: white;
}


/* tweak icons for treeviews */
treeview.image { color: alpha(currentColor, 0.8); }
treeview.image:selected { color: alpha(@theme_selected_fg_color, 0.9); }


/* utilities stack switcher */
ideeditorutilities > dzldockpaned > box > stackswitcher {
  margin: 6px;
}


/* buildui panel */
ideeditorsidebar notebook header {
  background: transparent;
}
ideeditorsidebar notebook header tab {
  padding: 0;
}

ideeditorproperties entry:last-child {
  border-radius: 0;
  border-right: none;
  border-left: none;
}


popover.messagepopover {
  padding: 0;
}
popover.messagepopover list {
  border-top: 1px solid @borders;
  border-radius: 0 0 5px 5px;
}
popover.messagepopover list row:not(:last-child) {
  border-bottom: 1px solid alpha(@borders, 0.3);
}
