@import url("resource:///org/gnome/builder/themes/Adwaita-shared.css");


perspectiveswitcher {
  background-color: #272c2e;
}
perspectiveswitcher button {
  color: #696c6b;
}
perspectiveswitcher button:checked,
perspectiveswitcher button:checked:hover {
  color: #eeeeec;
}
perspectiveswitcher button:hover {
  color: #b2b4b2;
}


entry.search-missing {
  border-color: #330000;
}

/* spellchecker word count background darker with dark theme */
.countbox {
  background-color: #587A9E;
}
