/* This file contains styling for various search widgets */
.search-frame {
  background-image: linear-gradient(shade(@theme_bg_color,1.05), @theme_bg_color);
  padding: 6px;
  border-style: solid;
  border-color: @borders;
  border-left-width: 1px;
  border-right-width: 1px;
  border-bottom-width: 1px;
  border-bottom-left-radius: 5px;
  border-bottom-right-radius: 5px;
}
frame.search-frame border {
  border: none;
}
.search-frame > box > grid:first-child > button.close:disabled,
.search-frame > box > grid:first-child > button.close {
  background: none;
  border: none;
  box-shadow: none;
  padding-left: 4px;
  padding-right: 4px;
}
.search-frame > box > grid:first-child > button.close image {
  color: @theme_fg_color;
  opacity: 0.3;
  margin: 2px;
  border: 1px solid transparent;
  border-radius: 3px;
}
.search-frame > box > grid:first-child > button.close:hover image {
  opacity: 0.75;
  transition-duration: 250ms;
  border: 1px solid @borders;
}
.search-frame > box > grid:first-child > button.close:active image {
  opacity: 0.8;
  background-image: linear-gradient(shade(@theme_bg_color, 0.9), @theme_bg_color);
}
.search-frame > box > grid:first-child > button.close:backdrop image {
  opacity: 0.1;
}
.search-occurrences-tag {
    color: shade (@theme_unfocused_fg_color, 0.8);
    border: 0px;
    margin: 2px;
    padding: 2px;
}
