.install-progress {
  background-image: linear-gradient(to top, @theme_selected_bg_color 2px, alpha(@theme_selected_bg_color, 0) 2px);
  background-repeat: no-repeat;
  background-position: 0 bottom;
  transition: none;
}

.install-progress:dir(rtl) {
  background-position: 100% bottom;
}
