/* ide-service.h
 *
 * Copyright (C) 2015 Christian Hergert <christian@hergert.me>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef IDE_SERVICE_H
#define IDE_SERVICE_H

#include "ide-object.h"

G_BEGIN_DECLS

#define IDE_TYPE_SERVICE (ide_service_get_type())

G_DECLARE_INTERFACE (IdeService, ide_service, IDE, SERVICE, IdeObject)

struct _IdeServiceInterface
{
  GTypeInterface parent_interface;

  void         (*context_loaded) (IdeService *service);
  const gchar *(*get_name)       (IdeService *service);
  void         (*start)          (IdeService *service);
  void         (*stop)           (IdeService *service);
};

const gchar *ide_service_get_name (IdeService *self);
void         ide_service_start    (IdeService *self);
void         ide_service_stop     (IdeService *self);

G_END_DECLS

#endif /* IDE_SERVICE_H */
