greeter flowboxchild {
  border-radius: 11px;
  outline-offset: -3px;
  -gtk-outline-radius: 9px;
  min-width: 164px;
  min-height: 164px;
}

greeter flowboxchild:selected label {
  color: white;
  font-weight: bold;
}
