@import url("resource:///org/gnome/builder/plugins/color-picker-plugin/themes/shared.css");

/* palettew widget dnd indicator */
gstylecolorpanel gstylepalettewidget.dnd {
  border-color: #4e9a06;
  background-color: #4e9a06;
}

