/* ide-autotools-project-miner.h
 *
 * Copyright (C) 2015 Christian Hergert <christian@hergert.me>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef IDE_AUTOTOOLS_PROJECT_MINER_H
#define IDE_AUTOTOOLS_PROJECT_MINER_H

#include <ide.h>

G_BEGIN_DECLS

#define IDE_TYPE_AUTOTOOLS_PROJECT_MINER (ide_autotools_project_miner_get_type())

G_DECLARE_FINAL_TYPE (IdeAutotoolsProjectMiner, ide_autotools_project_miner, IDE, AUTOTOOLS_PROJECT_MINER, GObject)

GFile *ide_autotools_project_miner_get_root_directory (IdeAutotoolsProjectMiner *self);
void   ide_autotools_project_miner_set_root_directory (IdeAutotoolsProjectMiner *self,
                                                       GFile                    *root_directory);

G_END_DECLS

#endif /* IDE_AUTOTOOLS_PROJECT_MINER_H */
